# Generated by Django 3.2.16 on 2023-10-25 07:50

import core.fields
import datetime
from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('claim', '0024_add_claim_restore_perms'),
    ]

    operations = [
        migrations.CreateModel(
            name='ClaimSamplingBatch',
            fields=[
                ('validity_from', core.fields.DateTimeField(db_column='ValidityFrom', default=datetime.datetime.now)),
                ('validity_to', core.fields.DateTimeField(blank=True, db_column='ValidityTo', null=True)),
                ('legacy_id', models.IntegerField(blank=True, db_column='LegacyID', null=True)),
                ('id', models.AutoField(db_column='ClaimSamplingBatchID', primary_key=True, serialize=False)),
                ('uuid', models.CharField(db_column='ClaimSamplingBatchUUID', default=uuid.uuid4, max_length=36, unique=True)),
                ('is_completed', models.BooleanField()),
                ('is_applied', models.BooleanField()),
                ('computed_value', models.JSONField(blank=True, db_column='ComputedValue', null=True)),
                ('assigned_value', models.JSONField(blank=True, db_column='AssignedValue', null=True)),
                ('assigned_to', models.ForeignKey(db_column='AssignedTo', on_delete=django.db.models.deletion.DO_NOTHING, related_name='AssignedTo', to='claim.claimadmin')),
                ('created_by', models.ForeignKey(db_column='CreatedBy', on_delete=django.db.models.deletion.DO_NOTHING, related_name='CreatedBy', to='claim.claimadmin')),
            ],
            options={
                'db_table': 'claim_ClaimSamplingBatch',
                'managed': True,
            },
        ),
        migrations.CreateModel(
            name='ClaimSamplingBatchAssignment',
            fields=[
                ('validity_from', core.fields.DateTimeField(db_column='ValidityFrom', default=datetime.datetime.now)),
                ('validity_to', core.fields.DateTimeField(blank=True, db_column='ValidityTo', null=True)),
                ('legacy_id', models.IntegerField(blank=True, db_column='LegacyID', null=True)),
                ('id', models.AutoField(db_column='ClaimSamplingBatchAssignmentID', primary_key=True, serialize=False)),
                ('uuid', models.CharField(db_column='ClaimSamplingBatchAssignmentUUID', default=uuid.uuid4, max_length=36, unique=True)),
                ('status', models.CharField(choices=[('R', 'Reviewed'), ('S', 'Skipped'), ('I', 'Idle')], default='I', max_length=2)),
                ('claim_batch_id', models.ForeignKey(db_column='ClaimSamplingBatchID', on_delete=django.db.models.deletion.DO_NOTHING, related_name='ClaimSamplingBatchID', to='claim_sampling.claimsamplingbatch')),
                ('claim_id', models.ForeignKey(db_column='ClaimID', on_delete=django.db.models.deletion.DO_NOTHING, related_name='ClaimID', to='claim.claim')),
            ],
            options={
                'db_table': 'claim_ClaimBatchAssignment',
                'managed': True,
            },
        ),
    ]
