# Generated by Django 4.2.11 on 2024-05-19 15:10

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("claim_sampling", "0003_initial"),
    ]

    operations = [
        migrations.RenameField(
            model_name="claimsamplingbatchassignment",
            old_name="claim_id",
            new_name="claim",
        ),
        migrations.RenameField(
            model_name="claimsamplingbatchassignment",
            old_name="claim_batch_id",
            new_name="claim_batch",
        ),
        migrations.RenameField(
            model_name="historicalclaimsamplingbatchassignment",
            old_name="claim_id",
            new_name="claim",
        ),
        migrations.RenameField(
            model_name="historicalclaimsamplingbatchassignment",
            old_name="claim_batch_id",
            new_name="claim_batch",
        ),
    ]
