# Generated by Django 4.2.10 on 2024-06-11 12:03

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('claim_sampling', '0004_rename_claim_id_claimsamplingbatchassignment_claim_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='claimsamplingbatchassignment',
            name='status',
            field=models.CharField(choices=[('S', 'Skipped'), ('I', 'Idle')], default='I', max_length=2),
        ),
        migrations.AlterField(
            model_name='historicalclaimsamplingbatchassignment',
            name='status',
            field=models.CharField(choices=[('S', 'Skipped'), ('I', 'Idle')], default='I', max_length=2),
        ),
    ]
