# Generated by Django 4.2.11 on 2024-05-19 14:49

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("claim_sampling", "0001_initial"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="claimsamplingbatchassignment",
            name="claim_batch_id",
        ),
        migrations.RemoveField(
            model_name="claimsamplingbatchassignment",
            name="claim_id",
        ),
        migrations.DeleteModel(
            name="ClaimSamplingBatch",
        ),
        migrations.DeleteModel(
            name="ClaimSamplingBatchAssignment",
        ),
    ]
